﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace TestLibrary
{
    /// <summary>
    /// 앨범 아트 디스플레이
    /// </summary>
    [DisplayName("앨범 아트 디스플레이")]
    [Description("CD 쥬얼 케이스 오버레이로 앨범 커버 아트웍을 표시한다.")]
    [ToolboxItem(true)]
    [TemplatePart(Name = "PART_AlbumArt", Type = typeof(Image))]
    public class AlbumArtDisplay : Control
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 앨범 아트 이미지 속성
        /// </summary>
        public static readonly DependencyProperty AlbumArtImageProperty = DependencyProperty.Register
        (
            "AlbumArtImage",
            typeof(ImageSource),
            typeof(AlbumArtDisplay),
            new UIPropertyMetadata
            (
                null,
                AlbumArtImagePropertyChangedCallback,
                CoerceAlbumArtImagePropertyCallback
            )
        );

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 그리기 객체
        /// </summary>
        private readonly DrawingVisual drawingVisual = new DrawingVisual();

        /// <summary>
        /// 앨범 아트 렌더링 타겟 비트맵
        /// </summary>
        private readonly RenderTargetBitmap albumArtRenderTargetBitmap = new RenderTargetBitmap(433, 379, 96, 96, PixelFormats.Pbgra32);

        /// <summary>
        /// 빈 경우 비트맵 이미지
        /// </summary>
        private readonly BitmapImage emptyBitmapImage = new BitmapImage(new Uri("pack://application:,,,/TestLibrary;component/Image/NoArtwork.png", UriKind.Absolute));

        /// <summary>
        /// 오버레이 비트맵 이미지
        /// </summary>
        private readonly BitmapImage overlayBitmapImage = new BitmapImage(new Uri("pack://application:,,,/TestLibrary;component/Image/Overlay.png", UriKind.Absolute));

        /// <summary>
        /// 언더레이 비트맵 이미지
        /// </summary>
        private readonly BitmapImage underlayBitmapImage = new BitmapImage(new Uri("pack://application:,,,/TestLibrary;component/Image/Underlay.png", UriKind.Absolute));

        /// <summary>
        /// 앨범 아트 이미지
        /// </summary>
        private Image albumArtImage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 앨범 아트 이미지 - AlbumArtImage

        /// <summary>
        /// 앨범 아트 이미지
        /// </summary>
        [Category("Common")]
        public ImageSource AlbumArtImage
        {
            get
            {
                return (ImageSource)GetValue(AlbumArtImageProperty);
            }
            set
            {
                SetValue(AlbumArtImageProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - AlbumArtDisplay()

        /// <summary>
        /// 생성자
        /// </summary>
        static AlbumArtDisplay()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(AlbumArtDisplay), new FrameworkPropertyMetadata(typeof(AlbumArtDisplay)));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 앨범 아트 이미지 속성 변경시 콜백 처리하기 - AlbumArtImagePropertyChangedCallback(d, e)

        /// <summary>
        /// 앨범 아트 이미지 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void AlbumArtImagePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            AlbumArtDisplay albumArtDisplay = d as AlbumArtDisplay;

            if(albumArtDisplay != null)
            {
                albumArtDisplay.OnAlbumArtImageChanged((ImageSource)e.OldValue, (ImageSource)e.NewValue);
            }
        }

        #endregion
        #region 앨범 아트 이미지 속성 강제시 콜백 처리하기 - CoerceAlbumArtImagePropertyCallback(d, value)

        /// <summary>
        /// 앨범 아트 이미지 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceAlbumArtImagePropertyCallback(DependencyObject d, object value)
        {
            AlbumArtDisplay albumArtDisplay = d as AlbumArtDisplay;

            if(albumArtDisplay != null)
            {
                return albumArtDisplay.OnCoerceAlbumArtImage((ImageSource)value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 템플리트 적용시 처리하기 - OnApplyTemplate()

        /// <summary>
        /// 템플리트 적용시 처리하기
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            this.albumArtImage = GetTemplateChild("PART_AlbumArt") as Image;

            this.albumArtImage.Source = this.albumArtRenderTargetBitmap;

            DrawAlbumArt();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 앨범 아트 이미지 변경시 처리하기 - OnAlbumArtImageChanged(oldValue, newValue)

        /// <summary>
        /// 앨범 아트 이미지 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnAlbumArtImageChanged(ImageSource oldValue, ImageSource newValue)
        {
            DrawAlbumArt();
        }

        #endregion
        #region 앨범 아트 이미지 강제시 처리하기 - OnCoerceAlbumArtImage(value)

        /// <summary>
        /// 앨범 아트 이미지 강제시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        protected virtual ImageSource OnCoerceAlbumArtImage(ImageSource value)
        {
            return value;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 앨범 아트 그리기 - DrawAlbumArt()

        /// <summary>
        /// 앨범 아트 그리기
        /// </summary>
        private void DrawAlbumArt()
        {
            this.albumArtRenderTargetBitmap.Clear();

            using(DrawingContext drawingContext = this.drawingVisual.RenderOpen())
            {
                if(AlbumArtImage == null)
                {

                    drawingContext.DrawImage(this.emptyBitmapImage, new Rect(0, 0, this.albumArtRenderTargetBitmap.Width, this.albumArtRenderTargetBitmap.Height));
                }
                else
                {
                    drawingContext.DrawImage(this.underlayBitmapImage, new Rect(0 , 0, this.albumArtRenderTargetBitmap.Width     , this.albumArtRenderTargetBitmap.Height     ));
                    drawingContext.DrawImage(AlbumArtImage           , new Rect(54, 5, this.albumArtRenderTargetBitmap.Width - 62, this.albumArtRenderTargetBitmap.Height - 10));
                    drawingContext.DrawImage(this.overlayBitmapImage , new Rect(0 , 0, this.albumArtRenderTargetBitmap.Width     , this.albumArtRenderTargetBitmap.Height     ));
                }
            }

            this.albumArtRenderTargetBitmap.Render(this.drawingVisual);
        }

        #endregion
    }
}