﻿using System;
using System.ComponentModel;

using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

using System.Windows;
using System.Windows.Data;

namespace TestLibrary
{
    /// <summary>
    /// 디지털 시계
    /// </summary>
    [DisplayName("디지털 시계")]
    [Description("디지털 LED 시계처럼 시간을 표시한다.")]
    [ToolboxItem(true)]   
    [TemplatePart(Name = "PART_ClockGrid", Type = typeof(Grid))]    
    public class DigitalClock : Control
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 시간 속성
        /// </summary>
        public static readonly DependencyProperty TimeProperty = DependencyProperty.Register
        (
            "Time",
            typeof(TimeSpan),
            typeof(DigitalClock),
            new UIPropertyMetadata(TimeSpan.Zero, TimePropertyChangedCallback, CoerceTimePropertyCallback)
        );

        /// <summary>
        /// 시 표시 여부
        /// </summary>
        public static readonly DependencyProperty ShowHoursProperty = DependencyProperty.Register
        (
            "ShowHours",
            typeof(bool),
            typeof(DigitalClock),
            new UIPropertyMetadata
            (
                true,
                ShowHoursPropertyChangedCallback,
                CoerceShowHoursPropertyCallback
            )
        );

        /// <summary>
        /// 서브 초 표시 여부 
        /// </summary>
        public static readonly DependencyProperty ShowSubSecondsProperty = DependencyProperty.Register
        (
            "ShowSubSeconds",
            typeof(bool),
            typeof(DigitalClock),
            new UIPropertyMetadata
            (
                false,
                ShowSubSecondsPropertyChangedCallback,
                CoerceShowSubSecondsPropertyCallback
            )
        );

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 시침 사각형 1
        /// </summary>
        private readonly Rectangle hourRectangle1 = new Rectangle();

        /// <summary>
        /// 시침 사각형 2
        /// </summary>
        private readonly Rectangle hourRectangle2 = new Rectangle();

        /// <summary>
        /// 분침 사각형 1
        /// </summary>
        private readonly Rectangle minuteRectangle1 = new Rectangle();

        /// <summary>
        /// 분침 사각형 2
        /// </summary>
        private readonly Rectangle minuteRectangle2 = new Rectangle();

        /// <summary>
        /// 초침 사각형 1
        /// </summary>
        private readonly Rectangle secondRectangle1 = new Rectangle();

        /// <summary>
        /// 초침 사각형 2
        /// </summary>
        private readonly Rectangle secondRectangle2 = new Rectangle();

        /// <summary>
        /// 분리자 사각형 1
        /// </summary>
        private readonly Rectangle separatorRectangle1 = new Rectangle();

        /// <summary>
        /// 분리자 사각형 2
        /// </summary>
        private readonly Rectangle separatorRectangle2 = new Rectangle();

        /// <summary>
        /// 소수점 사각형
        /// </summary>
        private readonly Rectangle decimalRectangle = new Rectangle();

        /// <summary>
        /// 서브 초 사각형 1
        /// </summary>
        private readonly Rectangle subSecondRectangle1 = new Rectangle();

        /// <summary>
        /// 서브 초 사각형 2
        /// </summary>
        private readonly Rectangle subSecondRectangle2 = new Rectangle();


        /// <summary>
        /// 1 비주얼 브러시
        /// </summary>
        private readonly VisualBrush oneVisualBrush = new VisualBrush();

        /// <summary>
        /// 2 비주얼 브러시
        /// </summary>
        private readonly VisualBrush twoVisualBrush = new VisualBrush();

        /// <summary>
        /// 3 비주얼 브러시
        /// </summary>
        private readonly VisualBrush threeVisualBrush = new VisualBrush();

        /// <summary>
        /// 4 비주얼 브러시
        /// </summary>
        private readonly VisualBrush fourVisualBrush = new VisualBrush();

        /// <summary>
        /// 5 비주얼 브러시
        /// </summary>
        private readonly VisualBrush fiveVisualBrush = new VisualBrush();

        /// <summary>
        /// 6 비주얼 브러시
        /// </summary>
        private readonly VisualBrush sixVisualBrush = new VisualBrush();

        /// <summary>
        /// 7 비주얼 브러시
        /// </summary>
        private readonly VisualBrush sevenVisualBrush = new VisualBrush();

        /// <summary>
        /// 8 비주얼 브러시
        /// </summary>
        private readonly VisualBrush eightVisualBrush = new VisualBrush();

        /// <summary>
        /// 9 비주얼 브러시
        /// </summary>
        private readonly VisualBrush nineVisualBrush = new VisualBrush();

        /// <summary>
        /// 0 비주얼 브러시
        /// </summary>
        private readonly VisualBrush zeroVisualBrush = new VisualBrush();

        /// <summary>
        /// 소수점 비주얼 브러시
        /// </summary>
        private readonly VisualBrush decimalVisualBrush = new VisualBrush();

        /// <summary>
        /// 콜론 비주얼 브러시
        /// </summary>
        private readonly VisualBrush colonVisualBrush = new VisualBrush();

        /// <summary>
        /// 시계 그리드
        /// </summary>
        private Grid clockGrid;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 시간 - Time

        /// <summary>
        /// 시간
        /// </summary>
        public TimeSpan Time
        {
            get
            {
                return (TimeSpan)GetValue(TimeProperty);
            }
            set
            {
                SetValue(TimeProperty, value);
            }
        }

        #endregion
        #region 시 표시 여부 - ShowHours

        /// <summary>
        /// 시 표시 여부
        /// </summary>
        public bool ShowHours
        {
            get
            {
                return (bool)GetValue(ShowHoursProperty);
            }
            set
            {
                SetValue(ShowHoursProperty, value);
            }
        }

        #endregion
        #region 서브 초 표시 여부 - ShowSubSeconds

        /// <summary>
        /// 서브 초 표시 여부
        /// </summary>
        public bool ShowSubSeconds
        {
            get
            {
                return (bool)GetValue(ShowSubSecondsProperty);
            }
            set
            {
                SetValue(ShowSubSecondsProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - DigitalClock()

        /// <summary>
        /// 생성자
        /// </summary>
        static DigitalClock()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(DigitalClock), new FrameworkPropertyMetadata(typeof(DigitalClock)));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 시간 속성 변경시 콜백 처리하기 - TimePropertyChangedCallback(d, e)

        /// <summary>
        /// 시간 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void TimePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                digitalClock.OnTimeChanged((TimeSpan)e.OldValue, (TimeSpan)e.NewValue);
            }
        }

        #endregion
        #region 시간 속성 강제시 콜백 처리하기 - CoerceTimePropertyCallback(d, value)

        /// <summary>
        /// 시간 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceTimePropertyCallback(DependencyObject d, object value)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                return digitalClock.OnCoerceTime((TimeSpan)value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        #region 시 표시 여부 속성 변경시 콜백 처리하기 - ShowHoursPropertyChangedCallback(d, e)

        /// <summary>
        /// 시 표시 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void ShowHoursPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                digitalClock.OnShowHoursChanged((bool)e.OldValue, (bool)e.NewValue);
            }
        }

        #endregion
        #region 시 표시 여부 속성 강제시 콜백 처리하기 - CoerceShowHoursPropertyCallback(d, value)

        /// <summary>
        /// 시 표시 여부 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceShowHoursPropertyCallback(DependencyObject d, object value)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                return digitalClock.OnCoerceShowHours((bool)value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        #region 서브 초 표시 여부 속성 변경시 콜백 처리하기 - ShowSubSecondsPropertyChangedCallback(d, e)

        /// <summary>
        /// 서브 초 표시 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void ShowSubSecondsPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                digitalClock.OnShowSubSecondsChanged((bool)e.OldValue, (bool)e.NewValue);
            }
        }

        #endregion
        #region 서브 초 표시 여부 속성 강제시 콜백 처리하기 - CoerceShowSubSecondsPropertyCallback(d, value)

        /// <summary>
        /// 서브 초 표시 여부 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceShowSubSecondsPropertyCallback(DependencyObject d, object value)
        {
            DigitalClock digitalClock = d as DigitalClock;

            if(digitalClock != null)
            {
                return digitalClock.OnCoerceShowSubSeconds((bool)value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 템플리트 적용시 처리하기 - OnApplyTemplate()

        /// <summary>
        /// 템플리트 적용시 처리하기
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            if(this.clockGrid != null)
            {
                this.clockGrid.Children.Clear();
            }

            this.clockGrid = GetTemplateChild("PART_ClockGrid") as Grid;

            this.oneVisualBrush.Visual     = FindResource("One") as Visual;
            this.oneVisualBrush.Stretch    = Stretch.Uniform;
            this.oneVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.oneVisualBrush, CachingHint.Cache);

            this.twoVisualBrush.Visual     = FindResource("Two") as Visual;
            this.twoVisualBrush.Stretch    = Stretch.Uniform;
            this.twoVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.twoVisualBrush, CachingHint.Cache);

            this.threeVisualBrush.Visual     = FindResource("Three") as Visual;
            this.threeVisualBrush.Stretch    = Stretch.Uniform;
            this.threeVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.threeVisualBrush, CachingHint.Cache);

            this.fourVisualBrush.Visual     = FindResource("Four") as Visual;
            this.fourVisualBrush.Stretch    = Stretch.Uniform;
            this.fourVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.fourVisualBrush, CachingHint.Cache);

            this.fiveVisualBrush.Visual     = FindResource("Five") as Visual;
            this.fiveVisualBrush.Stretch    = Stretch.Uniform;
            this.fiveVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.fiveVisualBrush, CachingHint.Cache);

            this.sixVisualBrush.Visual     = FindResource("Six") as Visual;
            this.sixVisualBrush.Stretch    = Stretch.Uniform;
            this.sixVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.sixVisualBrush, CachingHint.Cache);

            this.sevenVisualBrush.Visual     = FindResource("Seven") as Visual;
            this.sevenVisualBrush.Stretch    = Stretch.Uniform;
            this.sevenVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.sevenVisualBrush, CachingHint.Cache);

            this.eightVisualBrush.Visual     = FindResource("Eight") as Visual;
            this.eightVisualBrush.Stretch    = Stretch.Uniform;
            this.eightVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.eightVisualBrush, CachingHint.Cache);

            this.nineVisualBrush.Visual     = FindResource("Nine") as Visual;
            this.nineVisualBrush.Stretch    = Stretch.Uniform;
            this.nineVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.nineVisualBrush, CachingHint.Cache);

            this.zeroVisualBrush.Visual     = FindResource("Zero") as Visual;
            this.zeroVisualBrush.Stretch    = Stretch.Uniform;
            this.zeroVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.zeroVisualBrush, CachingHint.Cache);

            this.decimalVisualBrush.Visual     = FindResource("Decimal") as Visual;
            this.decimalVisualBrush.Stretch    = Stretch.Uniform;
            this.decimalVisualBrush.AlignmentY = AlignmentY.Bottom;

            RenderOptions.SetCachingHint(this.decimalVisualBrush, CachingHint.Cache);

            this.colonVisualBrush.Visual     = FindResource("Colon") as Visual;
            this.colonVisualBrush.Stretch    = Stretch.Uniform;
            this.colonVisualBrush.AlignmentY = AlignmentY.Center;

            RenderOptions.SetCachingHint(this.colonVisualBrush, CachingHint.Cache);

            this.separatorRectangle1.Fill = GetDigitVisualBrush(':');
            this.separatorRectangle2.Fill = GetDigitVisualBrush(':');
            this.decimalRectangle.Fill    = GetDigitVisualBrush('.');

            FormatClockLayout();

            SplitDigits();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 시간 변경시 처리하기 - OnTimeChanged(oldValue, newValue)

        /// <summary>
        /// 시간 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">이전 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnTimeChanged(TimeSpan oldValue, TimeSpan newValue)
        {
            SplitDigits();
        }

        #endregion
        #region 시간 강제시 처리하기 - OnCoerceTime(value)

        /// <summary>
        /// 시간 강제시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        protected virtual TimeSpan OnCoerceTime(TimeSpan value)
        {
            return value;
        }

        #endregion

        #region 시 표시 여부 변경시 처리하기 - OnShowHoursChanged(oldValue, newValue)

        /// <summary>
        /// 시 표시 여부 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">이전 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnShowHoursChanged(bool oldValue, bool newValue)
        {
            FormatClockLayout();

            SplitDigits();
        }

        #endregion
        #region 시 표시 여부 강제시 처리하기 - OnCoerceShowHours(value)

        /// <summary>
        /// 시 표시 여부 강제시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        protected virtual bool OnCoerceShowHours(bool value)
        {
            return value;
        }

        #endregion

        #region 서브 초 표시 여부 변경시 처리하기 - OnShowSubSecondsChanged(oldValue, newValue)

        /// <summary>
        /// 서브 초 표시 여부 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">이전 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnShowSubSecondsChanged(bool oldValue, bool newValue)
        {
            FormatClockLayout();

            SplitDigits();
        }

        #endregion
        #region 서브 초 표시 여부 강제시 처리하기 - OnCoerceShowSubSeconds(value)

        /// <summary>
        /// 서브 초 표시 여부 강제시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        protected virtual bool OnCoerceShowSubSeconds(bool value)
        {
            return value;
        }

        #endregion

        #region 템플리트 변경시 처리하기 - OnTemplateChanged(oldTemplate, newTemplate)

        /// <summary>
        /// 템플리트 변경시 처리하기
        /// </summary>
        /// <param name="oldTemplate">이전 템플리트</param>
        /// <param name="newTemplate">신규 템플리트</param>
        protected override void OnTemplateChanged(ControlTemplate oldTemplate, ControlTemplate newTemplate)
        {
            base.OnTemplateChanged(oldTemplate, newTemplate);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 시계 레이아웃 포맷하기 - FormatClockLayout()

        /// <summary>
        /// 시계 레이아웃 포맷하기
        /// </summary>
        private void FormatClockLayout()
        {
            if(this.clockGrid == null)
            {
                return;
            }

            ColumnDefinitionCollection columnDefinitionCollection = this.clockGrid.ColumnDefinitions;

            columnDefinitionCollection.Clear();

            this.clockGrid.Children.Clear();

            Thickness digitMargin = new Thickness(5.0d);
            int       columnIndex = 0;
            double    gridWidth   = 0;
            double    gridHeight  = 115;

            if(ShowHours)
            {
                ColumnDefinition hour1ColumnDefinition = new ColumnDefinition();

                hour1ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

                columnDefinitionCollection.Add(hour1ColumnDefinition);

                this.hourRectangle1.Margin = digitMargin;

                Grid.SetColumn(this.hourRectangle1, columnIndex);

                this.clockGrid.Children.Add(this.hourRectangle1);

                gridWidth += 60;

                columnIndex++;


                ColumnDefinition hour2ColumnDefinition = new ColumnDefinition();

                hour2ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

                columnDefinitionCollection.Add(hour2ColumnDefinition);

                this.hourRectangle2.Margin = digitMargin;

                Grid.SetColumn(this.hourRectangle2, columnIndex);

                this.clockGrid.Children.Add(this.hourRectangle2);

                gridWidth += 60;

                columnIndex++;


                ColumnDefinition seaparator1ColumnDefinition = new ColumnDefinition();

                seaparator1ColumnDefinition.Width = new GridLength(20, GridUnitType.Star);

                columnDefinitionCollection.Add(seaparator1ColumnDefinition);

                this.separatorRectangle1.Margin = digitMargin;

                Grid.SetColumn(this.separatorRectangle1, columnIndex);

                this.clockGrid.Children.Add(this.separatorRectangle1);

                gridWidth += 30;

                columnIndex++;
            }

            ColumnDefinition minute1ColumnDefinition = new ColumnDefinition();

            minute1ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

            columnDefinitionCollection.Add(minute1ColumnDefinition);

            this.minuteRectangle1.Margin = digitMargin;

            Grid.SetColumn(this.minuteRectangle1, columnIndex);

            this.clockGrid.Children.Add(this.minuteRectangle1);

            gridWidth += 60;

            columnIndex++;


            ColumnDefinition minute2ColumnDefinition = new ColumnDefinition();

            minute2ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

            columnDefinitionCollection.Add(minute2ColumnDefinition);

            this.minuteRectangle2.Margin = digitMargin;

            Grid.SetColumn(this.minuteRectangle2, columnIndex);

            this.clockGrid.Children.Add(this.minuteRectangle2);

            gridWidth += 60;

            columnIndex++;


            ColumnDefinition seaparator2ColumnDefinition = new ColumnDefinition();

            seaparator2ColumnDefinition.Width = new GridLength(20, GridUnitType.Star);

            columnDefinitionCollection.Add(seaparator2ColumnDefinition);

            this.separatorRectangle2.Margin = digitMargin;

            Grid.SetColumn(this.separatorRectangle2, columnIndex);

            this.clockGrid.Children.Add(this.separatorRectangle2);

            gridWidth += 30;

            columnIndex++;


            ColumnDefinition second1ColumnDefinition = new ColumnDefinition();

            second1ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

            columnDefinitionCollection.Add(second1ColumnDefinition);

            this.secondRectangle1.Margin = digitMargin;

            Grid.SetColumn(this.secondRectangle1, columnIndex);

            this.clockGrid.Children.Add(this.secondRectangle1);

            gridWidth += 60;

            columnIndex++;


            ColumnDefinition second2ColumnDefinition = new ColumnDefinition();

            second2ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

            columnDefinitionCollection.Add(second2ColumnDefinition);

            this.secondRectangle2.Margin = digitMargin;

            Grid.SetColumn(this.secondRectangle2, columnIndex);

            this.clockGrid.Children.Add(this.secondRectangle2);

            gridWidth += 60;

            columnIndex++;

            if(ShowSubSeconds)
            {
                ColumnDefinition decimalColumnDefinition = new ColumnDefinition();

                decimalColumnDefinition.Width = new GridLength(20, GridUnitType.Star);

                columnDefinitionCollection.Add(decimalColumnDefinition);

                this.decimalRectangle.Margin = digitMargin;

                Grid.SetColumn(this.decimalRectangle, columnIndex);

                this.clockGrid.Children.Add(this.decimalRectangle);

                gridWidth += 30;

                columnIndex++;


                ColumnDefinition subSecond1ColumnDefinition = new ColumnDefinition();

                subSecond1ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

                columnDefinitionCollection.Add(subSecond1ColumnDefinition);

                this.subSecondRectangle1.Margin = digitMargin;

                Grid.SetColumn(this.subSecondRectangle1, columnIndex);

                this.clockGrid.Children.Add(this.subSecondRectangle1);

                gridWidth += 60;

                columnIndex++;


                ColumnDefinition subSecond2ColumnDefinition = new ColumnDefinition();

                subSecond2ColumnDefinition.Width = new GridLength(50, GridUnitType.Star);

                columnDefinitionCollection.Add(subSecond2ColumnDefinition);

                this.subSecondRectangle2.Margin = digitMargin;

                Grid.SetColumn(this.subSecondRectangle2, columnIndex);

                this.clockGrid.Children.Add(this.subSecondRectangle2);

                gridWidth += 60;

                columnIndex++;
            }

            this.clockGrid.Width  = gridWidth;
            this.clockGrid.Height = gridHeight;
        }

        #endregion
        #region 숫자 분리하기 - SplitDigits()

        /// <summary>
        /// 숫자 분리하기
        /// </summary>
        private void SplitDigits()
        {
            if(this.clockGrid == null)
            {
                return;
            }

            const string TIME_FORMAT = "00";

            string hourString = Time.Hours.ToString(TIME_FORMAT);

            this.hourRectangle1.Fill = GetDigitVisualBrush(hourString[0]);
            this.hourRectangle2.Fill = GetDigitVisualBrush(hourString[1]);

            string minuteString = Time.Minutes.ToString(TIME_FORMAT);

            this.minuteRectangle1.Fill = GetDigitVisualBrush(minuteString[0]);
            this.minuteRectangle2.Fill = GetDigitVisualBrush(minuteString[1]);

            string secondString = Time.Seconds.ToString(TIME_FORMAT);

            this.secondRectangle1.Fill = GetDigitVisualBrush(secondString[0]);
            this.secondRectangle2.Fill = GetDigitVisualBrush(secondString[1]);

            string subSecondString = (Time.Milliseconds / 1000.0d).ToString("0.00");

            this.subSecondRectangle1.Fill = GetDigitVisualBrush(subSecondString[2]);
            this.subSecondRectangle2.Fill = GetDigitVisualBrush(subSecondString[3]);
        }

        #endregion
        #region 숫자 비주얼 브러시 구하기 - GetDigitVisualBrush(digit)

        /// <summary>
        /// 숫자 비주얼 브러시 구하기
        /// </summary>
        /// <param name="digit">숫자</param>
        /// <returns>숫자 비주얼 브러시</returns>
        private VisualBrush GetDigitVisualBrush(char digit)
        {
            switch(digit)
            {
                case '1' : return this.oneVisualBrush;
                case '2' : return this.twoVisualBrush;
                case '3' : return this.threeVisualBrush;
                case '4' : return this.fourVisualBrush;
                case '5' : return this.fiveVisualBrush;
                case '6' : return this.sixVisualBrush;
                case '7' : return this.sevenVisualBrush;
                case '8' : return this.eightVisualBrush;
                case '9' : return this.nineVisualBrush;
                case '0' : return this.zeroVisualBrush;
                case '.' : return this.decimalVisualBrush;
                case ':' : return this.colonVisualBrush;
            }

            return null;
        }

        #endregion
    }
}