﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;

namespace TestLibrary
{    
    /// <summary>
    /// 이퀄라이저
    /// </summary>
    [DisplayName("이퀄라이저")]
    [Description("밴드 주파수 증폭을 표시하고 편집한다.")]
    [ToolboxItem(true)]
    [TemplatePart(Name = "PART_EqualizerGrid", Type = typeof(Grid))]  
    public class Equalizer : Control
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 이퀄라이저 값 배열 속성
        /// </summary>
        public static readonly DependencyProperty EqualizerValueArrayProperty = DependencyProperty.Register
        (
            "EqualizerValueArray",
            typeof(float[]),
            typeof(Equalizer),
            new UIPropertyMetadata
            (
                new float[] { 0f, 0f, 0f, 0f, 0f, 0f, 0f },
                EqualizerValueArrayPropertyChangedCallback,
                CoerceEqualizerValueArrayPropertyCallback
            )
        );

        /// <summary>
        /// 밴드 카운트 속성
        /// </summary>
        public static readonly DependencyProperty BandCountProperty = DependencyProperty.Register
        (
            "BandCount",
            typeof(int),
            typeof(Equalizer),
            new UIPropertyMetadata
            (
                7,
                BandCountPropertyChangedCallback,
                CoerceBandCountPropertyCallback
            )
        );

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 슬라이더 리스트
        /// </summary>
        private readonly List<Slider> sliderList = new List<Slider>();

        /// <summary>
        /// 이퀄라이저 그리드
        /// </summary>
        private Grid equalizerGrid;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 이퀄라이저 값 배열 - EqualizerValueArray

        /// <summary>
        /// 이퀄라이저 값 배열
        /// </summary>
        public float[] EqualizerValueArray
        {
            get
            {
                return (float[])GetValue(EqualizerValueArrayProperty);
            }
            set
            {
                SetValue(EqualizerValueArrayProperty, value);
            }
        }

        #endregion
        #region 밴드 카운트 - BandCount

        /// <summary>
        /// 밴드 카운트
        /// </summary>
        public int BandCount
        {
            get
            {
                return (int)GetValue(BandCountProperty);
            }
            set
            {
                SetValue(BandCountProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 -Equalizer()

        /// <summary>
        /// 생성자
        /// </summary>
        static Equalizer()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(Equalizer), new FrameworkPropertyMetadata(typeof(Equalizer)));
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Equalizer()

        /// <summary>
        /// 생성자
        /// </summary>
        public Equalizer()
        {
            CreateSliders();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 이퀄라이저 값 배열 속성 변경시 콜백 처리하기 - EqualizerValueArrayPropertyChangedCallback(d, e)

        /// <summary>
        /// 이퀄라이저 값 배열 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void EqualizerValueArrayPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Equalizer equalizer = d as Equalizer;

            if(equalizer != null)
            {
                equalizer.OnEqualizerValueArrayChanged((float[])e.OldValue, (float[])e.NewValue);
            }
        }

        #endregion
        #region 이퀄라이저 값 배열 속성 강제시 콜백 처리하기 - CoerceEqualizerValueArrayPropertyCallback(d, value)

        /// <summary>
        /// 이퀄라이저 값 배열 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceEqualizerValueArrayPropertyCallback(DependencyObject d, object value)
        {
            Equalizer equalizer = d as Equalizer;

            if(equalizer != null)
            {
                return equalizer.OnCoerceEqualizerValueArray((float[])value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        #region 밴드 카운트 속성 변경시 콜백 처리하기 - BandCountPropertyChangedCallback(d, e)

        /// <summary>
        /// 밴드 카운트 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void BandCountPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            Equalizer equalizer = d as Equalizer;

            if(equalizer != null)
            {
                equalizer.OnBandCountChanged((int)e.OldValue, (int)e.NewValue);
            }
        }

        #endregion
        #region 밴드 카운트 속성 강제시 콜백 처리하기 - CoerceBandCountPropertyCallback(d, value)

        /// <summary>
        /// 밴드 카운트 속성 강제시 콜백 처리하기
        /// </summary>
        /// <param name="d">의존 객체</param>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        private static object CoerceBandCountPropertyCallback(DependencyObject d, object value)
        {
            Equalizer equalizer = d as Equalizer;

            if(equalizer != null)
            {
                return equalizer.OnCoerceBancCount((int)value);
            }
            else
            {
                return value;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 템플리트 적용시 처리하기 - OnApplyTemplate()

        /// <summary>
        /// 템플리트 적용시 처리하기
        /// </summary>
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            if(this.equalizerGrid != null)
            {
                this.equalizerGrid.Children.Clear();
            }

            this.equalizerGrid = GetTemplateChild("PART_EqualizerGrid") as Grid;

            CreateSliders();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 이퀄라이저 값 배열 변경시 처리하기 - OnEqualizerValueArrayChanged(oldValueArray, newValueArray)

        /// <summary>
        /// 이퀄라이저 값 배열 변경시 처리하기
        /// </summary>
        /// <param name="oldValueArray">이전 값 배열</param>
        /// <param name="newValueArray">신규 값 배열</param>
        protected virtual void OnEqualizerValueArrayChanged(float[] oldValueArray, float[] newValueArray)
        {
            if(newValueArray == null || newValueArray.Length != BandCount)
            {
                SetEqualizerValueArray(new float[BandCount]);
            }
            else
            {
                SetEqualizerValueArray(newValueArray);
            }
        }

        #endregion
        #region 이퀄라이저 값 배열 강제시 처리하기 - OnCoerceEqualizerValueArray(valueArray)

        /// <summary>
        /// 이퀄라이저 값 배열 강제시 처리하기
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        /// <returns>강제 값 배열</returns>
        protected virtual float[] OnCoerceEqualizerValueArray(float[] valueArray)
        {
            if(valueArray == null || valueArray.Length != BandCount)
            {
                return new float[BandCount];
            }

            return valueArray;
        }

        #endregion

        #region 밴드 속성 강제시 처리하기 - OnCoerceBancCount( value)

        /// <summary>
        /// 밴드 속성 강제시 처리하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>강제 값</returns>
        protected virtual int OnCoerceBancCount(int value)
        {
            value = Math.Max(1, value);

            return value;
        }

        #endregion
        #region 밴드 속성 변경시 처리하기 - OnBandCountChanged(oldValue, newValue)

        /// <summary>
        /// 밴드 속성 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">이전 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnBandCountChanged(int oldValue, int newValue)
        {
            CreateSliders();

            SetEqualizerValueArray(new float[BandCount]);
        }

        #endregion

        #region 템플리트 변경시 처리하기 - OnTemplateChanged(oldTemplate, newTemplate)

        /// <summary>
        /// 템플리트 변경시 처리하기
        /// </summary>
        /// <param name="oldTemplate">이전 템플리트</param>
        /// <param name="newTemplate">신규 템플리트</param>
        protected override void OnTemplateChanged(ControlTemplate oldTemplate, ControlTemplate newTemplate)
        {
            base.OnTemplateChanged(oldTemplate, newTemplate);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private
        ////////////////////////////////////////////////////////////////////// Event

        #region 슬라이더 값 변경시 처리하기 - slider_ValueChanged(sender, e)

        /// <summary>
        /// 슬라이더 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            EqualizerValueArray = GetEqualizerValueArray();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////// Function

        #region 슬라이더 지우기 - ClearSliders()

        /// <summary>
        /// 슬라이더 지우기
        /// </summary>
        private void ClearSliders()
        {
            foreach(Slider slider in this.sliderList)
            {
                slider.ValueChanged -= slider_ValueChanged;

                this.equalizerGrid.Children.Remove(slider);
            }

            this.sliderList.Clear();
        }

        #endregion
        #region 슬라이더 생성하기 - CreateSliders()

        /// <summary>
        /// 슬라이더 생성하기
        /// </summary>
        private void CreateSliders()
        {
            if(this.equalizerGrid == null)
            {
                return;
            }

            ClearSliders();

            this.equalizerGrid.ColumnDefinitions.Clear();

            for(int i = 0; i < BandCount; i++)
            {
                ColumnDefinition columnDefinition = new ColumnDefinition()
                {
                    Width = new GridLength(10.0d, GridUnitType.Star)
                };

                this.equalizerGrid.ColumnDefinitions.Add(columnDefinition);

                Slider slider = new Slider()
                {
                    Style       = (Style)FindResource("EqualizerSlider"),
                    Maximum     = 1.0,
                    Minimum     = -1.0,
                    SmallChange = 0.01,
                    LargeChange = 0.1,
                };

                Grid.SetColumn(slider, i);

                this.sliderList.Add(slider);

                this.equalizerGrid.Children.Add(slider);

                slider.ValueChanged += slider_ValueChanged;
            }
        }

        #endregion

        #region 이퀄라이저 값 배열 구하기 - GetEqualizerValueArray()

        /// <summary>
        /// 이퀄라이저 값 배열 구하기
        /// </summary>
        /// <returns>이퀄라이저 값 배열</returns>
        private float[] GetEqualizerValueArray()
        {
            float[] valueArray = new float[BandCount];

            for(int i = 0; i < BandCount; i++)
            {
                valueArray[i] = (float)this.sliderList[i].Value;
            }

            return valueArray;
        }

        #endregion
        #region 이퀄라이저 값 배열 설정하기 - SetEqualizerValueArray(valueArray)

        /// <summary>
        /// 이퀄라이저 값 배열 설정하기
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        private void SetEqualizerValueArray(float[] valueArray)
        {
            for(int i = 0; i < BandCount; i++)
            {
                this.sliderList[i].Value = valueArray[i];
            }
        }

        #endregion
    }
}