﻿using System;

using NAudio.Dsp;

namespace TestProject
{
    /// <summary>
    /// 샘플 수집기
    /// </summary>
    public class SampleAggregator
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 볼륨 왼쪽 최소값
        /// </summary>
        private float volumeLeftMinimumValue;

        /// <summary>
        /// 볼륨 왼쪽 최대값
        /// </summary>
        private float volumeLeftMaximumValue;

        /// <summary>
        /// 볼륨 오른쪽 최소값
        /// </summary>
        private float volumeRightMinimumValue;

        /// <summary>
        /// 볼륨 오른쪽 최대값
        /// </summary>
        private float volumeRightMaximumValue;

        /// <summary>
        /// 채널 복소수 배열
        /// </summary>
        private Complex[] channelComplexArray;

        /// <summary>
        /// 채널 복소수 배열 인덱스
        /// </summary>
        private int channelComplexArrayIndex;

        /// <summary>
        /// 버퍼 크기
        /// </summary>
        private int bufferSize;

        /// <summary>
        /// 이진 지수
        /// </summary>
        private int binaryExponentation;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 왼쪽 최소 볼륨 - LeftMinimumVolume

        /// <summary>
        /// 왼쪽 최소 볼륨
        /// </summary>
        public float LeftMinimumVolume
        {
            get
            {
                return this.volumeLeftMinimumValue;
            }
        }

        #endregion
        #region 왼쪽 최대 볼륨 - LeftMaximumVolume

        /// <summary>
        /// 왼쪽 최대 볼륨
        /// </summary>
        public float LeftMaximumVolume
        {
            get
            {
                return this.volumeLeftMaximumValue;
            }
        }

        #endregion
        #region 오른쪽 최소 볼륨 - RightMinimumVolume

        /// <summary>
        /// 오른쪽 최소 볼륨
        /// </summary>
        public float RightMinimumVolume
        {
            get
            {
                return this.volumeRightMinimumValue;
            }
        }

        #endregion
        #region 오른쪽 최대 볼륨 - RightMaximumVolume

        /// <summary>
        /// 오른쪽 최대 볼륨
        /// </summary>
        public float RightMaximumVolume
        {
            get
            {
                return this.volumeRightMaximumValue;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SampleAggregator(bufferSize)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="bufferSize">버퍼 크기</param>
        public SampleAggregator(int bufferSize)
        {
            this.bufferSize          = bufferSize;
            this.binaryExponentation = (int)Math.Log(bufferSize, 2);
            this.channelComplexArray = new Complex[bufferSize];
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 지우기 - Clear()

        /// <summary>
        /// 지우기
        /// </summary>
        public void Clear()
        {
            this.volumeLeftMinimumValue   = float.MaxValue;
            this.volumeLeftMaximumValue   = float.MinValue;
            this.volumeRightMinimumValue  = float.MaxValue;
            this.volumeRightMaximumValue  = float.MinValue;
            this.channelComplexArrayIndex = 0;
        }

        #endregion
        #region 추가하기 - Add(leftValue, rightValue)

        /// <summary>
        /// 추가하기
        /// </summary>
        /// <param name="leftValue">왼쪽 값</param>
        /// <param name="rightValue">오른쪽 값</param>
        public void Add(float leftValue, float rightValue)
        {            
            if(this.channelComplexArrayIndex == 0)
            {
                this.volumeLeftMinimumValue  = float.MaxValue;
                this.volumeLeftMaximumValue  = float.MinValue;
                this.volumeRightMinimumValue = float.MaxValue;
                this.volumeRightMaximumValue = float.MinValue;
            }

            this.channelComplexArray[this.channelComplexArrayIndex].X = (leftValue + rightValue) / 2.0f;
            this.channelComplexArray[this.channelComplexArrayIndex].Y = 0;

            this.channelComplexArrayIndex++;            

            this.volumeLeftMinimumValue  = Math.Min(this.volumeLeftMinimumValue , leftValue );
            this.volumeLeftMaximumValue  = Math.Max(this.volumeLeftMaximumValue , leftValue );
            this.volumeRightMinimumValue = Math.Min(this.volumeRightMinimumValue, rightValue);
            this.volumeRightMaximumValue = Math.Max(this.volumeRightMaximumValue, rightValue);

            if(this.channelComplexArrayIndex >= this.channelComplexArray.Length)
            {
                this.channelComplexArrayIndex = 0;
            }
        }

        #endregion
        #region FFT 데이터 설정하기 - SetFFTData(targetArray)

        /// <summary>
        /// FFT 데이터 설정하기
        /// </summary>
        /// <param name="targetArray">타겟 배열</param>
        public void SetFFTData(float[] targetArray)
        {
            Complex[] copyComplexArray = new Complex[this.bufferSize];

            this.channelComplexArray.CopyTo(copyComplexArray, 0);

            FastFourierTransform.FFT(true, this.binaryExponentation, copyComplexArray);

            for(int i = 0; i < copyComplexArray.Length / 2; i++)
            {
                targetArray[i] = (float)Math.Sqrt(copyComplexArray[i].X * copyComplexArray[i].X + copyComplexArray[i].Y * copyComplexArray[i].Y);
            }
        }

        #endregion
    }
}