﻿using System.Windows;
using System.Windows.Data;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// UI 헬퍼
    /// </summary>
    public static class UIHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 바인딩하기 - Bind(dataSource, sourcePath, targetObject, dependencyProperty, stringFormat, bindingMode, converter)

        /// <summary>
        /// 바인딩하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        /// <param name="sourcePath">소스 경로</param>
        /// <param name="targetObject">타겟 객체</param>
        /// <param name="dependencyProperty">의존 속성</param>
        /// <param name="stringFormat">문자열 포맷</param>
        /// <param name="bindingMode">바인딩 모드</param>
        /// <param name="converter">컨버터</param>
        public static void Bind(object dataSource, string sourcePath, FrameworkElement targetObject, DependencyProperty dependencyProperty, string stringFormat, BindingMode bindingMode, IValueConverter converter)
        {
            Binding binding = new Binding
            {
                Source              = dataSource,
                UpdateSourceTrigger = UpdateSourceTrigger.PropertyChanged,
                Mode                = bindingMode,
                Path                = new PropertyPath(sourcePath),
                StringFormat        = stringFormat,
                Converter           = converter
            };

            targetObject.SetBinding(dependencyProperty, binding);
        }

        #endregion
        #region 바인딩하기 - Bind(dataSource, sourcePath, targetObject, dependencyProperty)

        /// <summary>
        /// 바인딩하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        /// <param name="sourcePath">소스 경로</param>
        /// <param name="targetObject">타겟 객체</param>
        /// <param name="dependencyProperty">의존 속성</param>
        public static void Bind(object dataSource, string sourcePath, FrameworkElement targetObject, DependencyProperty dependencyProperty)
        {
            Bind(dataSource, sourcePath, targetObject, dependencyProperty, null, BindingMode.Default, null);
        }

        #endregion
        #region 바인딩하기 - Bind(dataSource, sourcePath, targetObject, dependencyProperty, stringFormat)

        /// <summary>
        /// 바인딩하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        /// <param name="sourcePath">소스 경로</param>
        /// <param name="targetObject">타겟 객체</param>
        /// <param name="dependencyProperty">의존 속성</param>
        /// <param name="stringFormat">문자열 포맷</param>
        public static void Bind(object dataSource, string sourcePath, FrameworkElement targetObject, DependencyProperty dependencyProperty, string stringFormat)
        {
            Bind(dataSource, sourcePath, targetObject, dependencyProperty, stringFormat, BindingMode.Default, null);
        }

        #endregion
        #region 바인딩하기 - Bind(dataSource, sourcePath, targetObject, dependencyProperty, bindingMode)

        /// <summary>
        /// 바인딩하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        /// <param name="sourcePath">소스 경로</param>
        /// <param name="targetObject">타겟 객체</param>
        /// <param name="dependencyProperty">의존 속성</param>
        /// <param name="bindingMode">바인딩 모드</param>
        public static void Bind(object dataSource, string sourcePath, FrameworkElement targetObject, DependencyProperty dependencyProperty, BindingMode bindingMode)
        {
            Bind(dataSource, sourcePath, targetObject, dependencyProperty, null, bindingMode, null);
        }

        #endregion
        #region 바인딩하기 - Bind(dataSource, sourcePath, targetObject, dependencyProperty, stringFormat, bindingMode)

        /// <summary>
        /// 바인딩하기
        /// </summary>
        /// <param name="dataSource">데이터 소스</param>
        /// <param name="sourcePath">소스 경로</param>
        /// <param name="targetObject">타겟 객체</param>
        /// <param name="dependencyProperty">의존 속성</param>
        /// <param name="stringFormat">문자열 포맷</param>
        /// <param name="bindingMode">바인딩 포맷</param>
        public static void Bind(object dataSource, string sourcePath, FrameworkElement targetObject, DependencyProperty dependencyProperty, string stringFormat, BindingMode bindingMode)
        {
            Bind(dataSource, sourcePath, targetObject, dependencyProperty, stringFormat, bindingMode, null);
        }

        #endregion

        #region 부모 의존 객체 찾기 - FindParent<TParent>(child)

        /// <summary>
        /// 부모 의존 객체 찾기
        /// </summary>
        /// <typeparam name="TParent">부모 타입</typeparam>
        /// <param name="child">자식</param>
        /// <returns>부모</returns>
        public static TParent FindParent<TParent>(this DependencyObject child) where TParent : DependencyObject
        {
            DependencyObject parent = VisualTreeHelper.GetParent(child);

            do
            {
                TParent parentMatched = parent as TParent;

                if(parentMatched != null)
                {
                    return parentMatched;
                }

                parent = VisualTreeHelper.GetParent(parent);
            }
            while(parent != null);

            return null;
        }

        #endregion
    }
}